@@ -10,16 +10,17 @@ __author__ = 'Carlos Alberto Gomez Gonzalez, V. Christiaens'
 __all__ = ['frame_fix_badpix_isolated',
            'cube_fix_badpix_isolated',
            'cube_fix_badpix_annuli',
-           'cube_fix_badpix_clump']
+           'cube_fix_badpix_clump',
+           'cube_fix_badpix_with_kernel']
 
 import numpy as np
 from skimage.draw import disk, ellipse
 from scipy.ndimage import median_filter
 from astropy.stats import sigma_clipped_stats
 from ..stats import sigma_filter
-from ..var import frame_center, get_annulus_segments
+from ..var import frame_center, get_annulus_segments, frame_filter_lowpass
 from ..stats import clip_array
-from ..conf import timing, time_ini, Progressbar
+from ..config import timing, time_ini, Progressbar
 from .cosmetics import approx_stellar_position
 
 import warnings
@@ -32,45 +33,58 @@ except ImportError:
     no_numba = True
 
 def frame_fix_badpix_isolated(array, bpm_mask=None, sigma_clip=3, num_neig=5,
-                              size=5, protect_mask=False, radius=30,
-                              verbose=True):
+                              size=5, protect_mask=0, cxy=None, mad=False, 
+                              ignore_nan=True, verbose=True, full_output=False):
     """ Corrects the bad pixels, marked in the bad pixel mask. The bad pixel is
-     replaced by the median of the adjacent pixels. This function is very fast
-     but works only with isolated (sparse) pixels.
+    replaced by the median of the adjacent pixels. This function is very fast
+    but works only with isolated (sparse) pixels.
 
-     Parameters
-     ----------
-     array : numpy ndarray
-         Input 2d array.
-     bpm_mask : numpy ndarray, optional
-         Input bad pixel map. Zeros frame where the bad pixels have a value of
-         1. If None is provided a bad pixel map will be created using
-         sigma clip statistics. 
-     sigma_clip : int, optional
-         In case no bad pixel mask is provided all the pixels above and below
-         sigma_clip*STDDEV will be marked as bad.
-     num_neig : int, optional
-         The side of the square window around each pixel where the sigma clipped
-         statistics are calculated (STDDEV and MEDIAN). If the value is equal to
-         0 then the statistics are computed in the whole frame.
-     size : odd int, optional
-         The size the box (size x size) of adjacent pixels for the median
-         filter.
-     protect_mask : bool, optional
-         If True a circular aperture at the center of the frames will be
-         protected from any operation. With this we protect the star and
-         vicinity.
-     radius : int, optional
-         Radius of the circular aperture (at the center of the frames) for the
-         protection mask.
-     verbose : bool, optional
-         If True additional information will be printed.
-
-     Return
-     ------
-     frame : numpy ndarray
-         Frame with bad pixels corrected.
-     """
+    Parameters
+    ----------
+    array : numpy ndarray
+        Input 2d array.
+    bpm_mask : numpy ndarray, optional
+        Input bad pixel map. Zeros frame where the bad pixels have a value of
+        1. If None is provided a bad pixel map will be created using
+        sigma clip statistics. 
+    sigma_clip : int, optional
+        In case no bad pixel mask is provided all the pixels above and below
+        sigma_clip*STDDEV will be marked as bad.
+    num_neig : int, optional
+        The side of the square window around each pixel where the sigma clipped
+        statistics are calculated (STDDEV and MEDIAN). If the value is equal to
+        0 then the statistics are computed in the whole frame.
+    size : odd int, optional
+        The size the box (size x size) of adjacent pixels for the median
+        filter.
+    protect_mask : int or float, optional
+        If larger than 0, radius of a circular aperture (at the center of the 
+        frames) in which no bad pixels will be identified. This can be useful 
+        to protect the star and vicinity.
+    cxy: None or tuple
+        If protect_mask is True, this is the location of the star centroid in
+        the images. If None, assumes the star is already centered. If a tuple,
+        the location of the star is assumed to be the same in all frames of the 
+        cube.
+    mad : {False, True}, bool optional
+        If True, the median absolute deviation will be used instead of the 
+        standard deviation.  
+    ignore_nan: bool, optional
+        Whether to not consider NaN values as bad pixels. If False, will also 
+        correct them.
+    verbose : bool, optional
+        If True additional information will be printed.
+    full_output: bool, {False,True}, optional
+        Whether to return as well the cube of bad pixel maps and the cube of 
+        defined annuli.
+        
+    Return
+    ------
+    frame : numpy ndarray
+        Frame with bad pixels corrected.
+    bpm_mask: 2d array
+        The bad pixel map
+    """
     if array.ndim != 2:
         raise TypeError('Array is not a 2d array or single frame')
     if size % 2 == 0:
@@ -87,14 +101,21 @@ def frame_fix_badpix_isolated(array, bpm_mask=None, sigma_clip=3, num_neig=5,
         neigh = False
 
     frame = array.copy()
-    cy, cx = frame_center(frame)
+    if cxy is None:
+        cy, cx = frame_center(frame)
+    else:
+        cx, cy = cxy
+        
     if bpm_mask is None:
+        ori_nan_mask = np.where(np.isnan(frame))
         ind = clip_array(frame, sigma_clip, sigma_clip, neighbor=neigh,
-                         num_neighbor=num_neig, mad=True)
+                         num_neighbor=num_neig, mad=mad)
         bpm_mask = np.zeros_like(frame)
         bpm_mask[ind] = 1
+        if ignore_nan:
+            bpm_mask[ori_nan_mask] = 0
         if protect_mask:
-            cir = disk((cy, cx), radius)
+            cir = disk((cy, cx), protect_mask, shape=bpm_mask.shape)
             bpm_mask[cir] = 0
         bpm_mask = bpm_mask.astype('bool')
 
@@ -107,12 +128,17 @@ def frame_fix_badpix_isolated(array, bpm_mask=None, sigma_clip=3, num_neig=5,
         msg = "/nDone replacing {} bad pixels using the median of neighbors"
         print(msg.format(count_bp))
         timing(start)
-    return array_out
+        
+    if full_output:
+        return array_out, bpm_mask
+    else:
+        return array_out
 
 
 def cube_fix_badpix_isolated(array, bpm_mask=None, sigma_clip=3, num_neig=5, 
-                             size=5, frame_by_frame=False, protect_mask=False, 
-                             radius=30, verbose=True):
+                             size=5, frame_by_frame=False, protect_mask=0, 
+                             cxy=None, mad=False, ignore_nan=True, verbose=True, 
+                             full_output=False):
     """ Corrects the bad pixels, marked in the bad pixel mask. The bad pixel is 
     replaced by the median of the adjacent pixels. This function is very fast
     but works only with isolated (sparse) pixels. 
@@ -138,20 +164,34 @@ def cube_fix_badpix_isolated(array, bpm_mask=None, sigma_clip=3, num_neig=5,
         Whether to correct bad pixels frame by frame in the cube. By default it
         is set to False; the bad pixels are computed on the mean frame of the 
         stack (faster but not necessarily optimal).
-    protect_mask : bool, optional
-        If True a circular aperture at the center of the frames will be 
-        protected from any operation. With this we protect the star and its
-        vicinity.
-    radius : int, optional 
-        Radius of the circular aperture (at the center of the frames) for the 
-        protection mask.
+    protect_mask : int or float, optional
+        If larger than 0, radius of a circular aperture (at the center of the 
+        frames) in which no bad pixels will be identified. This can be useful 
+        to protect the star and vicinity.
+    cxy: None, tuple or 2d numpy ndarray
+        If protect_mask is True, this is the location of the star centroid in
+        the images. If None, assumes the star is already centered. If a tuple,
+        the location of the star is assumed to be the same in all frames of the 
+        cube. If a (n_frames x 2) ndarray, it should contain the xy location of 
+        the star in each frame.
+    mad : {False, True}, bool optional
+        If True, the median absolute deviation will be used instead of the 
+        standard deviation.  
+    ignore_nan: bool, optional
+        Whether to not consider NaN values as bad pixels. If False, will also 
+        correct them.
     verbose : bool, optional
         If True additional information will be printed.
-    
+    full_output: bool, {False,True}, optional
+        Whether to return as well the cube of bad pixel maps and the cube of 
+        defined annuli.
+        
     Return
     ------
     array_out : numpy ndarray
         Cube with bad pixels corrected.
+    bpm_mask: 2d or 3d array
+        The bad pixel map or the cube of bpix maps
     """
     if array.ndim != 3:
         raise TypeError('Array is not a 3d array or cube')
@@ -168,57 +208,86 @@ def cube_fix_badpix_isolated(array, bpm_mask=None, sigma_clip=3, num_neig=5,
     else:
         neigh = False
     
-    cy, cx = frame_center(array[0])
+    nz = array.shape[0]
+        
+    if cxy is None:
+        cy, cx = frame_center(array[0])
+    elif isinstance(cxy, tuple):
+        cx, cy = cxy
+    elif isinstance(cxy, np.ndarray):
+        if cxy.shape[0] != nz or cxy.shape[1] != 2 or cxy.ndim != 2:
+            raise ValueError("cxy does not have right shape")
+        elif not frame_by_frame:
+            msg = "cxy must be a tuple or None if not in frame_by_frame mode"
+            raise ValueError(msg)
+        else:
+            cx = cxy[:,0]
+            cy = cxy[:,1]
+
+            
     array_out = array.copy()
+    final_bpm = np.zeros_like(array_out, dtype=bool)
     n_frames = array.shape[0]
     count_bp = 0
     if frame_by_frame:
+        if np.isscalar(cx):
+            cx = [cx]*nz
+            cy = [cy]*nz
         for i in Progressbar(range(n_frames), desc="processing frames"):
-            array_out[i] = frame_fix_badpix_isolated(array[i], bpm_mask=bpm_mask, 
-                                                    sigma_clip=sigma_clip, 
-                                                    num_neig=num_neig,
-                                                    size=size, 
-                                                    protect_mask=protect_mask, 
-                                                    radius=radius,
-                                                    verbose=False,
-                                                    debug=False)
-            if verbose:                                       
-                bpm = np.where(array_out[i]!=array[i])   
-                count_bp+=np.sum(np.ones_like(array_out[i])[bpm])                                   
+            res = frame_fix_badpix_isolated(array[i], bpm_mask=bpm_mask, 
+                                            sigma_clip=sigma_clip,
+                                            num_neig=num_neig, size=size, 
+                                            protect_mask=protect_mask, 
+                                            verbose=False, cxy=(cx[i],cy[i]), 
+                                            ignore_nan=ignore_nan,
+                                            full_output=True)
+            array_out[i] = res[0]
+            final_bpm[i] = res[1]     
+        count_bp = np.sum(final_bpm)                           
     else:                                                
         if bpm_mask is None:
-            ind = clip_array(np.mean(array, axis=0), sigma_clip, sigma_clip,
-                             neighbor=neigh, num_neighbor=num_neig, mad=True)
-            bpm_mask = np.zeros_like(array[0])
-            bpm_mask[ind] = 1
+            ori_nan_mask = np.where(np.isnan(np.nanmean(array, axis=0)))
+            ind = clip_array(np.nanmean(array, axis=0), sigma_clip, sigma_clip,
+                             neighbor=neigh, num_neighbor=num_neig, mad=mad)
+            final_bpm = np.zeros_like(array[0], dtype=bool)
+            final_bpm[ind] = 1
+            if ignore_nan:
+                final_bpm[ori_nan_mask] = 0
             if protect_mask:
-                cir = disk((cy, cx), radius)
-                bpm_mask[cir] = 0
-            bpm_mask = bpm_mask.astype('bool')
+                cir = disk((cy, cx), protect_mask, shape=final_bpm.shape)
+                final_bpm[cir] = 0
+            final_bpm = final_bpm.astype('bool')
     
         for i in Progressbar(range(n_frames), desc="processing frames"):
             frame = array_out[i]
             smoothed = median_filter(frame, size, mode='mirror')
-            frame[np.where(bpm_mask)] = smoothed[np.where(bpm_mask)]
+            frame[np.where(final_bpm)] = smoothed[np.where(final_bpm)]
             if verbose: 
-                count_bp+=np.sum(bpm_mask)  
+                count_bp+=np.sum(final_bpm)  
             
     if verbose: 
-        msg = "/nDone replacing {} bad pixels using the median of neighbors"
+        msg = "Done replacing {:.0f} bad pixels using the median of neighbors"
         print(msg.format(count_bp))
+        if not frame_by_frame:
+            msg = "(i.e. {:.0f} static bad pixels per channel))"
+            print(msg.format(count_bp/n_frames))            
         timing(start)
-    return array_out
+        
+    if full_output:
+        return array_out, final_bpm
+    else:
+        return array_out
 
 
 def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5., 
-                           protect_psf=True, r_in_std=10, r_out_std=None, 
+                           protect_mask=0, r_in_std=10, r_out_std=None,
                            verbose=True, half_res_y=False, min_thr=None, 
                            max_thr=None, full_output=False):
     """
     Function to correct the bad pixels annulus per annulus (centered on the 
     provided location of the star), in an input frame or cube.
-    This function is MUCH FASTER than bp_clump_removal (about 20 times faster);
-    hence to be prefered in all cases where there is only one bright source.
+    This function is faster than bp_clump_removal; hence to be prefered in all 
+    cases where there is only one bright source with circularly symmetric PSF.
     The bad pixel values are replaced by: ann_median + ann_stddev*random_gauss;
     where ann_median is the median of the annulus, ann_stddev is the standard 
     deviation in the annulus, and random_gauss is a random factor picked from a 
@@ -241,11 +310,10 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
     sig: Float scalar, optional
         Number of stddev above or below the median of the pixels in the same 
         annulus, to consider a pixel as bad.
-    protect_psf: bool, {True, False}, optional
-        Whether to protect a circular region centered on the star (1.8*fwhm 
-        radius) from any bpix corr. If False, there is a risk of modifying a 
-        centroid peak value if it is too "peaky"; but if True real bad pixels 
-        within the core are not corrected.
+    protect_mask : int or float, optional
+        If larger than 0, radius of a circular aperture (at the center of the 
+        frames) in which no bad pixels will be identified. This can be useful 
+        to protect the star and vicinity.
     r_in_std: float, optional
         Inner radius in fwhm for the calculation of the standard 
         deviation of the background - used for min threshold 
@@ -271,10 +339,13 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
 
     Returns:
     --------
-    obj_tmp: 2d or 3d array; the bad pixel corrected frame/cube.
+    obj_tmp: 2d or 3d array
+        The bad pixel corrected frame/cube.
     If full_output is set to True, it returns as well:
-    bpix_map: 2d or 3d array; the bad pixel map or the cube of bpix maps
-    ann_frame_cumul: 2 or 3d array; the cube of defined annuli
+    bpix_map: 2d or 3d array
+        The bad pixel map or the cube of bpix maps
+    ann_frame_cumul: 2 or 3d array
+        The cube of defined annuli
     """
 
     obj_tmp = array.copy()
@@ -287,7 +358,7 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
     if max_thr is None:
         max_thr = np.amax(obj_tmp)-1
 
-    def bp_removal_2d(obj_tmp, cy, cx, fwhm, sig, protect_psf, r_in_std,
+    def bp_removal_2d(obj_tmp, cy, cx, fwhm, sig, protect_mask, r_in_std,
                       r_out_std, verbose):
 
         n_x = obj_tmp.shape[1]
@@ -353,12 +424,12 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
                 rr_big = rr
                 rr_sma= rr
             if half_res_y:
-                big_ell_idx = ellipse(cy=cy, cx=cx, 
+                big_ell_idx = ellipse(r=cy, c=cx, 
                                       r_radius=((rr_big+1)*ann_width)/2, 
                                       c_radius=(rr_big+1)*ann_width, 
                                       shape=(n_y,n_x))
                 if rr != 0:
-                    small_ell_idx = ellipse(cy=cy, cx=cx, 
+                    small_ell_idx = ellipse(r=cy, c=cx, 
                                             r_radius=(rr_sma*ann_width)/2, 
                                             c_radius=rr_sma*ann_width, 
                                             shape=(n_y,n_x))
@@ -402,12 +473,12 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
         
         #3/ Create a tuple-array with coordinates of a circle of radius 1.8*fwhm
         # centered on the provided coordinates of the star
-        if protect_psf:
+        if protect_mask:
             if half_res_y: 
-                circl_new = ellipse(cy, cx, r_radius=0.9*fwhm, 
-                                    c_radius=1.8*fwhm, shape=(n_y,n_x))
+                circl_new = ellipse(cy, cx, r_radius=protect_mask/2., 
+                                    c_radius=protect_mask, shape=(n_y,n_x))
             else: 
-                circl_new = disk((cy, cx), radius=1.8*fwhm, 
+                circl_new = disk((cy, cx), radius=protect_mask, 
                                  shape=(n_y, n_x))
         else: circl_new = []
 
@@ -452,7 +523,7 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
             cx = cen[0,1]
         obj_tmp, bpix_map, ann_frame_cumul = bp_removal_2d(obj_tmp, cy, cx, 
                                                            fwhm, sig, 
-                                                           protect_psf, 
+                                                           protect_mask, 
                                                            r_in_std, r_out_std,
                                                            verbose)
     if ndims == 3:
@@ -473,7 +544,7 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
                 print('************Frame # ', i,' *************')
                 print('centroid assumed at coords:',cx[i],cy[i])    
             res_i = bp_removal_2d(obj_tmp[i], cy[i], cx[i], fwhm[i], sig,
-                                  protect_psf, r_in_std, r_out_std, verbose)
+                                  protect_mask, r_in_std, r_out_std, verbose)
             obj_tmp[i], bpix_map[i], ann_frame_cumul[i] = res_i
  
     if full_output:
@@ -483,15 +554,16 @@ def cube_fix_badpix_annuli(array, fwhm, cy=None, cx=None, sig=5.,
 
 
 def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4., 
-                          sig=4., protect_psf=True, verbose=True, 
-                          half_res_y=False, min_thr=None, max_nit=15, 
+                          sig=4., protect_mask=0, verbose=True, 
+                          half_res_y=False, min_thr=None, max_nit=15, mad=True,
                           full_output=False):
     """
-    Function to correct clumps of bad pixels. Very fast when a bad pixel map is 
-    provided. If a bad pixel map is not provided, the bad pixel clumps will be 
-    searched iteratively and replaced by the median of good neighbouring pixel 
-    values if enough of them. The size of the box is set by the closest odd 
-    integer larger than fwhm (to avoid accidentally replacing a companion).
+    Function to identify and correct clumps of bad pixels. Very fast when a bad 
+    pixel map is provided. If a bad pixel map is not provided, the bad pixel 
+    clumps will be searched iteratively and replaced by the median of good 
+    neighbouring pixel values, when enough of them are available. The size of 
+    the box is set by the closest odd integer larger than fwhm (to avoid 
+    accidentally replacing point sources).
 
 
 
@@ -515,11 +587,10 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
         Value representing the number of "sigmas" above or below the "median" of
         the neighbouring pixel, to consider a pixel as bad. See details on 
         parameter "m" of function reject_outlier.
-    protect_psf: bool, {True, False}, optional
-        True if you want to protect a circular region centered on the star 
-        (1.8*fwhm radius) from any bpix corr. If False, there is a risk to 
-        modify a psf peak value; but if True, real bpix within the core are 
-        not corrected.
+    protect_mask : int or float, optional
+        If larger than 0, radius of a circular aperture (at the center of the 
+        frames) in which no bad pixels will be identified. This can be useful 
+        to protect the star and vicinity.
     verbose: bool, {False,True}, optional
         Whether to print the number of bad pixels and number of iterations 
         required for each frame.
@@ -529,25 +600,32 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
         there are always 2 rows of pixels with exactly the same values.
         The algorithm will just consider every other row (hence making it
         twice faster), then apply the bad pixel correction on all rows.
-    min_thr: float or None, opt
-        If provided, corresponds to a minimum absolute threshold below which
-        pixels are not considered bad (can be used to avoid the identification
-        of bad pixels within noise).
+    min_thr: float, tuple or None, opt
+        If a float is provided, corresponds to a minimum absolute threshold 
+        below which pixels are not considered bad (can be used to avoid the 
+        identification of bad pixels within noise).
+        If a tuple of 2 values, corresponds to the range of values within which 
+        not to consider a pixel as bad. (e.g. (-0.1, 10.)).
     max_nit: float, optional
         Maximum number of iterations on a frame to correct bpix. Typically, it 
         should be set to less than ny/2 or nx/2. This is a mean of precaution in
         case the algorithm gets stuck with 2 neighbouring pixels considered bpix
         alternately on two consecutively iterations hence leading to an infinite
         loop (very very rare case).
+    mad : {False, True}, bool optional
+        If True, the median absolute deviation will be used instead of the 
+        standard deviation.
     full_output: bool, {False,True}, optional
         Whether to return as well the cube of bad pixel maps and the cube of 
         defined annuli.
 
     Returns:
     --------
-    obj_tmp: 2d or 3d array; the bad pixel corrected frame/cube.
+    obj_tmp: 2d or 3d array
+        the bad pixel corrected frame/cube.
     If full_output is set to True, it returns as well:
-    bpix_map: 2d or 3d array; the bad pixel map or the cube of bpix maps
+    bpix_map: 2d or 3d array
+        the bad pixel map or the cube of bpix maps
     """
 
     obj_tmp = array.copy()
@@ -558,7 +636,8 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
         if bpm_mask.shape[-2:] != array.shape[-2:]:
             raise TypeError("Bad pixel map has wrong y/x dimensions.")
 
-    def bp_removal_2d(obj_tmp, cy, cx, fwhm, sig, protect_psf, min_thr, verbose):    
+    def bp_removal_2d(obj_tmp, cy, cx, fwhm, sig, protect_mask, min_thr, 
+                      half_res_y, mad, verbose):    
         n_x = obj_tmp.shape[1]
         n_y = obj_tmp.shape[0]
 
@@ -584,24 +663,36 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
         
         #1/ Create a tuple-array with coordinates of a circle of radius 1.8*fwhm
         # centered on the approximate coordinates of the star
-        if protect_psf:
+        if protect_mask:
             if half_res_y: 
-                circl_new = ellipse(int(cy/2), cx, r_radius=0.9*fwhm, 
-                                    c_radius=1.8*fwhm, shape=(n_y, n_x))
-            else: circl_new = disk((cy, cx), radius=1.8*fwhm, 
+                circl_new = ellipse(int(cy/2), cx, r_radius=0.5*protect_mask, 
+                                    c_radius=protect_mask, shape=(n_y, n_x))
+            else: circl_new = disk((cy, cx), radius=protect_mask, 
                                    shape=(n_y, n_x))
         else: circl_new = []
     
 
         #3/ Create a bad pixel map, by detecting them with clip_array
         bp=clip_array(obj_tmp, sig, sig, out_good=False, neighbor=True,
-                      num_neighbor=neighbor_box, mad=True)
+                      num_neighbor=neighbor_box, mad=mad, 
+                      half_res_y=half_res_y)
         bpix_map = np.zeros_like(obj_tmp)  
-        bpix_map[bp] = 1              
+        bpix_map[bp] = 1
+        if min_thr is not None:
+            if np.isscalar(min_thr):
+                min_thr = (-min_thr, min_thr)
+            elif not isinstance(min_thr, tuple):
+                msg = "if provided, min_thr should be float or tuple"
+                raise ValueError(msg)
+            else:
+                if len(min_thr) != 2:
+                   msg = "if min_thr is a tuple, it should have 2 elements"
+                   raise ValueError(msg) 
+            cond1 = obj_tmp>min_thr[0]
+            cond2 = obj_tmp<min_thr[1]
+            bpix_map[np.where(cond1 & cond2)] = 0
         nbpix_tot = np.sum(bpix_map)
         bpix_map[circl_new] = 0
-        if min_thr is not None:
-            bpix_map[np.where(np.abs(obj_tmp)<min_thr)] = 0
         nbpix_tbc = np.sum(bpix_map)
         bpix_map_cumul = np.zeros_like(bpix_map)
         bpix_map_cumul[:] = bpix_map[:]
@@ -614,15 +705,19 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
                 print("Iteration {}: {} bpix in total, {} to be "
                       "corrected".format(nit, nbpix_tot, nbpix_tbc))
             obj_tmp = sigma_filter(obj_tmp, bpix_map, neighbor_box=neighbor_box,
-                                   min_neighbors=nneig, verbose=verbose)
+                                   min_neighbors=nneig, half_res_y=half_res_y, 
+                                   verbose=verbose)
             bp=clip_array(obj_tmp, sig, sig, out_good=False, neighbor=True,
-                          num_neighbor=neighbor_box, mad=True)
+                          num_neighbor=neighbor_box, mad=mad, 
+                          half_res_y=half_res_y)
             bpix_map = np.zeros_like(obj_tmp)  
             bpix_map[bp] = 1
+            if min_thr is not None:
+                cond1 = obj_tmp>min_thr[0]
+                cond2 = obj_tmp<min_thr[1]
+                bpix_map[np.where(cond1 & cond2)] = 0
             nbpix_tot = np.sum(bpix_map)
             bpix_map[circl_new] = 0
-            if min_thr is not None:
-                bpix_map[np.where(np.abs(obj_tmp)<min_thr)] = 0
             nbpix_tbc = np.sum(bpix_map)
             bpix_map_cumul = bpix_map_cumul+bpix_map
 
@@ -643,20 +738,20 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
 
     if ndims == 2:
         if bpm_mask is None:
-            if (cy is None or cx is None) and protect_psf:
+            if (cy is None or cx is None) and protect_mask:
                 cen = approx_stellar_position([obj_tmp], fwhm)
                 cy = cen[0,0]
                 cx = cen[0,1]
             obj_tmp, bpix_map_cumul = bp_removal_2d(obj_tmp, cy, cx, fwhm, sig, 
-                                                    protect_psf, min_thr, 
-                                                    verbose)
+                                                    protect_mask, min_thr, 
+                                                    half_res_y, mad, verbose)
         else:
             fwhm_round = int(round(fwhm))
             fwhm_round = fwhm_round+1-(fwhm_round%2) # make it odd
             neighbor_box = max(3, fwhm_round) # to not replace a companion
             nneig = sum(np.arange(3, neighbor_box+2, 2))
             obj_tmp = sigma_filter(obj_tmp, bpm_mask, neighbor_box, nneig, 
-                                   verbose)
+                                   half_res_y, verbose)
             bpix_map_cumul = bpm_mask
                                             
     if ndims == 3:
@@ -675,9 +770,11 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
             for i in range(n_z):
                 if verbose: print('************Frame # ', i,' *************')
                 obj_tmp[i], bpix_map_cumul[i] = bp_removal_2d(obj_tmp[i], cy[i], 
-                                                              cx[i], fwhm[i], sig, 
-                                                              protect_psf,  
-                                                              min_thr, verbose)
+                                                              cx[i], fwhm[i], 
+                                                              sig, protect_mask,  
+                                                              min_thr, 
+                                                              half_res_y, mad,
+                                                              verbose)
         else:
             if isinstance(fwhm, (float,int)):
                 fwhm_round = int(round(fwhm))
@@ -693,7 +790,7 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
                 else:
                     bpm = bpm_mask
                 obj_tmp[i] = sigma_filter(obj_tmp[i], bpm, neighbor_box, 
-                                          nneig, verbose)
+                                          nneig, half_res_y, verbose)
             bpix_map_cumul = bpm_mask
                                                  
     if full_output:
@@ -702,6 +799,154 @@ def cube_fix_badpix_clump(array, bpm_mask=None, cy=None, cx=None, fwhm=4.,
         return obj_tmp
     
     
+def cube_fix_badpix_with_kernel(array, bpm_mask, mode='gauss', fwhm=4., 
+                                kernel_sz=None, psf=None, half_res_y=False,
+                                **kwargs):
+    """
+    Function to correct clumps of bad pixels by interpolation with a 
+    user-defined kernel (through astropy.convolution). A bad pixel map must be 
+    provided (e.g. found with function `cube_fix_badpix_clump`).
+
+
+    Parameters
+    ----------
+    array : 3D or 2D array 
+        Input 3d cube or 2d image.
+    bpix_map: 3D or 2D array
+        Input bad pixel array. Should have same x,y dimenstions as array. 
+        If 2D, but input array is 3D, the same bpix_map will be assumed for all 
+        frames.
+    mode: str, optional {'gauss', 'psf'}
+        Can be either a 2D Gaussian ('gauss') or an input normalized PSF 
+        ('psf').
+    fwhm: float or 1D array, opt
+        If mode is 'gauss', the fwhm of the Gaussian.
+    kernel_sz: int or None, optional
+        Size of the kernel in pixels for 2D Gaussian and Moffat convolutions.
+        If None, astropy.convolution will automatically consider 8*radius 
+        kernel sizes.
+    psf: 2D or 3D array, optional
+        If mode is 'psf', a normalized PSF array. If a 3D cube is provided 
+        (e.g. for spectral cubes), the first dimension should match that of the 
+        input array (which should also be 3D). Else, the same 2D PSF kernel 
+        will be for all input frames, whether the input is 2D or 3D.
+        If half_res_y is True, psf should be provided vertically squashed.
+    half_res_y: bool, {True,False}, optional
+        Whether the input data has only half the angular resolution vertically 
+        compared to horizontally (e.g. the case for some IFUs); in other words
+        there are always 2 rows of pixels with exactly the same values.
+        If so, the Gaussian kernel will also be squashed vertically by a 
+        factor 2.
+    **kwargs : dict
+        Passed through to the astropy.convolution.convolve or convolve_fft
+        function.
+
+    Returns:
+    --------
+    obj_tmp: 2d or 3d array; the bad pixel corrected frame/cube.
+    """
+    
+    obj_tmp = array.copy()
+    ndims = obj_tmp.ndim
+    assert ndims == 2 or ndims == 3, "Object is not two or three dimensional.\n"
+
+    if bpm_mask.shape[-2:] != array.shape[-2:]:
+        raise TypeError("Bad pixel map has wrong y/x dimensions.")
+        
+    if np.sum(bpm_mask) == 0:
+        msg = "Warning: no bad pixel found in bad pixel map. "
+        msg += "Returning input array as is."
+        print(msg)
+        return obj_tmp
+        
+    ny, nx = obj_tmp.shape[-2:]
+    if ndims == 3:
+        nz = obj_tmp.shape[0]
+        
+    if bpm_mask.ndim == 2 and ndims == 3:
+        master_bpm = np.zeros([nz, ny, nx])
+        for z in range(nz):
+            master_bpm[z] = bpm_mask[np.newaxis,:,:]
+        bpm_mask = master_bpm.copy()
+    
+    # below: superseded by mask option?
+    # first replace all bad pixels with NaN values - they will be interpolated
+    # obj_tmp[np.where(bpm_mask)] = np.nan
+    
+    if half_res_y:
+        # squash vertically
+        def squash_v(array):
+            ny, nx = array.shape
+            if ny%2:
+                raise ValueError("Input array y dimension should be even")
+            nny = ny//2
+            new_obj_tmp = np.zeros([nny, nx])
+            for y in range(nny):
+                new_obj_tmp[y] = array[int(y*2)]
+            return new_obj_tmp
+            
+        if ndims == 2:
+            obj_tmp = squash_v(obj_tmp)
+            bpm_mask = squash_v(bpm_mask)
+        else:
+            new_obj_tmp = []
+            new_bpm_mask = []
+            for z in range(nz):
+                new_obj_tmp.append(squash_v(obj_tmp[z]))
+                new_bpm_mask.append(squash_v(bpm_mask[z]))
+            obj_tmp = np.array(new_obj_tmp)
+            bpm_mask = np.array(new_bpm_mask)
+    
+    if ndims == 2:     
+        obj_tmp_corr = frame_filter_lowpass(obj_tmp, mode=mode, fwhm_size=fwhm,
+                                            conv_mode='convfft', 
+                                            kernel_sz=kernel_sz, psf=psf, 
+                                            mask=bpm_mask, iterate=True, 
+                                            half_res_y=half_res_y, **kwargs)
+    else:
+        obj_tmp_corr = obj_tmp.copy()
+        if np.isscalar(fwhm):
+            fwhm = [fwhm]*nz
+        if psf is None:
+            psf = [psf]*nz
+        elif psf.ndim==2:
+            psf = [psf]*nz
+        elif psf.shape[0] != nz:
+            raise ValueError("input psf must have same z dimension as array")
+        for z in range(nz):
+            obj_tmp_corr[z] = frame_filter_lowpass(obj_tmp[z], mode=mode, 
+                                                   fwhm_size=fwhm[z],
+                                                   conv_mode='convfft', 
+                                                   kernel_sz=kernel_sz,
+                                                   psf=psf[z], mask=bpm_mask[z],
+                                                   iterate=True, 
+                                                   half_res_y=half_res_y, 
+                                                   **kwargs)
+    
+    ## replace only the bad pixels (obj_tmp_corr is low-pass filtered)
+    obj_tmp[np.where(bpm_mask)] = obj_tmp_corr[np.where(bpm_mask)]
+    
+    if half_res_y:
+        # unsquash vertically
+        def unsquash_v(array):
+            ny, nx = array.shape
+            nny = int(ny*2)
+            new_obj_tmp = np.zeros([nny, nx])
+            for y in range(nny):
+                new_obj_tmp[y] = array[y//2]
+            return new_obj_tmp
+        if ndims == 2:
+            obj_tmp = unsquash_v(obj_tmp)
+        else:
+            new_obj_tmp = []
+            for z in range(nz):
+                new_obj_tmp.append(unsquash_v(obj_tmp[z]))
+            obj_tmp = np.array(new_obj_tmp)
+    
+    return obj_tmp
+
+    
+    
 def find_outliers(frame, sig_dist, in_bpix=None, stddev=None, neighbor_box=3,
                   min_thr=None, mid_thr=None):
     """ Provides a bad pixel (or outlier) map for a given frame.
@@ -977,12 +1222,12 @@ def correct_ann_outliers(obj_tmp, ann_width, sig, med_neig, std_neig, cy, cx,
         Boolean array with location of outliers.
     """ 
     
-    if no_numba: 
+    if True:#no_numba: 
         def _correct_ann_outliers(obj_tmp, ann_width, sig, med_neig, std_neig, 
                                   cy, cx, min_thr, max_thr, rand_arr, stddev, 
                                   half_res_y=False):           
             n_y, n_x = obj_tmp.shape
-            rand_arr = 2*(np.random.rand((n_y, n_x))-0.5)
+            rand_arr = 2*(np.random.rand(n_y, n_x)-0.5)
             obj_tmp_corr = obj_tmp.copy()
             bpix_map = np.zeros([n_y,n_x])
             for yy in range(n_y):
@@ -1018,7 +1263,7 @@ def correct_ann_outliers(obj_tmp, ann_width, sig, med_neig, std_neig, cy, cx,
                                   cy, cx, min_thr, max_thr, rand_arr, stddev, 
                                   half_res_y=False):           
             n_y, n_x = obj_tmp.shape
-            rand_arr = 2*(np.random.rand((n_y, n_x))-0.5)
+            rand_arr = 2*(np.random.rand(n_y, n_x)-0.5)
             obj_tmp_corr = obj_tmp.copy()
             bpix_map = np.zeros([n_y,n_x])
             for yy in range(n_y):
